/*
  Name : base.cpp
  Copyright : KPAMEGAN & ROUSSET
  Authors : KPAMEGAN Karel, ROUSSET Baptiste
  Description: file which contents function to use a LCD screen
*/

#include "base.hpp"
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <iostream>
#include <unistd.h>

using namespace std;

int fdAll=0;               //global file descriptor
int Screen::counter=0;     //counter of screen

/*
  Name : Screen
  Finality : this is a constructor
  Decription : initialize the device with the one puts on argument,
	       if it is the first implemented screen, the device of the lcd is open,
               we initialize the file descriptor,
	       we initialize the lines of the screen to empty lines
*/
Screen::Screen(char *dev)
{
  fd=fdAll;
  device=dev;
  strcpy(line0,"                   ");
  strcpy(line1,"                   ");
  strcpy(line2,"                   ");
  strcpy(line3,"                   ");
  if(counter==0)
    open_screen();
  ++counter;
}

/*
  Name : Screen
  Finality : this is a constructor
  Decription : initialize the device to "/dev/lcd" ( the lcd device ),
	       if it is the first implemented screen, the device of the lcd is open,
               we initialize the file descriptor,
	       we initialize the lines of the screen to empty lines
*/
Screen::Screen()
{
  fd=fdAll;
  device="/dev/lcd";
  strcpy(line0,"                   ");
  strcpy(line1,"                   ");
  strcpy(line2,"                   ");
  strcpy(line3,"                   ");
  if(counter==0)
    open_screen();
  ++counter;
}

/*
  Name : open_screen
  Finality : open the device of the screen
  Decription : open the file of the device,
               delete the screen and set the cursor to the home position,
	       ( maximum on the top and maximum on the left )
*/
void Screen::open_screen(void)
{
  if((fd=open(device,O_WRONLY))==-1)
    perror("open constructor : Are you sure that you are ROOT ?"),exit(0);
  fdAll=fd;
  delete_screen();
  init_home_cursor();
}

/*
  Name : close_screen
  Finality : close the device of the screen
  Decription : close the device of the screen
*/
void Screen::close_screen(void)
{
  close(fd);
  fd=-1;
}

/*
  Name : ~Screen
  Finality : delete the screen
  Decription : initialise the cursor to the home position ( maximum on the top and maximum on the left ),
               if it is the only screen we close the screen,
	       decreas the couter of screen 
*/
Screen:: ~Screen()
{
  init_home_cursor();
  if(counter==0)
    close_screen();
  --counter;
}

/*
  Name : init_home_cursor
  Finality : initialize to the home position
  Decription : set the cursor to the home position ( maximum on the top and maximum on the left )
*/
int Screen::init_home_cursor(void)
{
  if((write(fd,"\033[H",3))==-1)
    return -1;
  return 0;
}

/*
  Name : set_cursor_line
  Finality : set the cursor to the line which is given on argument
  Decription : set the cursor to the home position,
               move the cursor down as such as wanted
*/
int Screen::set_cursor_line(int nol)
{
  if((init_home_cursor())<0)
    return -2;
  if((go_down(nol))<0)
    return -3;
  return 0;
}

/*
  Name : delete_line
  Finality : delete the line passed on argument
  Decription : set the cursor to the line to delete
               delete the line
*/
int Screen::delete_line(int nol)
{
  if((set_cursor_line(nol))<0)
    return -2;
  if((write(fd,"\033[K",3))==-1)
    return -1;
  return 0;
}

/*
  Name : delete_screen
  Finality : delete the screen
  Decription : delete the screen
*/
int Screen::delete_screen(void)
{
  if((write(fd,"\033[J",3))==-1)
    return -1;
  return 0;
}

/*
  Name : get_line
  Finality : return the line which is wanted
  Decription : choose the line wanted and return it
*/
char *Screen::get_line(int nol)
{
  switch(nol)
    {
    case 0 : return line0;break;
    case 1 : return line1;break;
    case 2 : return line2;break;
    case 3 : return line3;break;
    }
  return NULL;
}

/*
  Name : get_screen
  Finality : return the screen address
  Decription : return the screen address
*/
Screen *Screen::get_screen(void)
{
  return this;
}

/*
  Name : set_line
  Finality : set characters into a line 
  Decription : selection the line to change
               copy characters given on argument into the selectionned line
*/
int Screen::set_line(int nol, char *newLine)
{
  int i=0,l=0;
  l=strlen(newLine);
  switch(nol)
    {
    case 0 : strcpy(line0,newLine);
      for(i=l;i<20;i++)
	line0[i]=' ';
      break;
    case 1 : strcpy(line1,newLine);
      for(i=l;i<20;i++)
	line1[i]=' ';
      break;
    case 2 : strcpy(line2,newLine);
      for(i=l;i<20;i++)
	line2[i]=' ';
      break;
    case 3 : strcpy(line3,newLine);
      for(i=l;i<20;i++)
	line3[i]=' ';
      break;
    }
   return 0;
}

/*
  Name : show
  Finality : display the screen
  Decription : print lines of the screen
*/
int Screen::show(void)
{
  if((set_cursor_line(0))<0)
    return -2;
  if((write(fd,line0,20))==-1)
    return -1;
  if((set_cursor_line(1))<0)
    return -2;
  if((write(fd,line1,20))==-1)
    return -1;
  if((set_cursor_line(2))<0)
    return -2;
  if((write(fd,line2,20))==-1)
    return -1;
  if((set_cursor_line(3))<0)
    return -2;
  if((write(fd,line3,20))==-1)
    return -1;
  return 0;
}

/*
  Name : set_line_center
  Finality : set characters into the center of a line 
  Decription : selection the line to change
               copy characters given on argument into the selectionned line on the center
*/
int Screen::set_line_center(int nol,char *newLine)
{
  int l=strlen(newLine);
  int start=10-(l/2);
  int i=0;
  int rest=(l+start);
  char tab[20]="                   ";
  switch(nol)
    {
    case 0 : strcpy(line0,tab);
             strcpy(&line0[start],newLine);
	     for(i=rest;i<20;i++)
	       line0[i]=' ';
	     break;
    case 1 : strcpy(line1,tab);
             strcpy(&line1[start],newLine);
	     for(i=rest;i<20;i++)
	       line1[i]=' ';
	     break;
    case 2 : strcpy(line2,tab);
             strcpy(&line2[start],newLine);
	     for(i=rest;i<20;i++)
	       line2[i]=' ';
	     break;
    case 3 : strcpy(line3,tab);
             strcpy(&line3[start],newLine);
	     for(i=rest;i<20;i++)
	       line3[i]=' ';
	     break;
    }
  return 0;
}

/*
  Name : set_screen
  Finality : copy a screen
  Decription : copy the contents of lines of the screen given on argument 
*/
int Screen::set_screen(Screen *newScreen)
{
  set_line(0,newScreen->line0);
  set_line(1,newScreen->line1);
  set_line(2,newScreen->line2);
  set_line(3,newScreen->line3);
  return 0;
}

/*
  Name : set_stars
  Finality : set stars on the left and on the left of all the screen
  Decription : insert into the begin and the end of each lines a star( * )
*/
void Screen::set_stars(void)
{
 line0[0]=line0[19]=line1[0]=line1[19]=line2[0]=line2[19]=line3[0]=line3[19]='*';
}

/*
  Name : set_separators
  Finality : set separators between two half-screens
  Decription : insert into the middle of each lines two pipes( || )
*/
void Screen::set_separators(void)
{
 line0[9]='|'; line0[10]='|';
line1[9]='|'; line1[10]='|';
line2[9]='|'; line2[10]='|';
line3[9]='|'; line3[10]='|';
 //set_line_center(0,"||");
  //set_line_center(1,"||");
  //set_line_center(2,"||");
  //set_line_center(3,"||");
}

/*
  Name : go_down
  Finality : put the cursor down
  Decription : put the cursor down of #nb line(s)
*/
int Screen::go_down(int nb)
{
  while(nb>0)
    {
      if((write(fd,"\033[B",3))==-1)
	return -1;
      nb--;
    }
  return 0;
}

/*
  Name : go_up
  Finality : put the cursor up
  Decription : put the cursor up of #nb line(s)
*/
int Screen::go_up(int nb)
{
  while(nb>0)
    {
      if((write(fd,"\033[A",3))==-1)
	return -1;
      nb--;
    }
  return 0;
}

/*
  Name : go_right
  Finality : put the cursor right
  Decription : put the cursor right of #nb line(s)
*/
int Screen::go_right(int nb)
{
  while(nb>0)
    {
      if((write(fd,"\033[C",3))==-1)
	return -1;
      nb--;
    }
  return 0;
}

/*
  Name : go_left
  Finality : put the cursor left
  Decription : put the cursor left of #nb line(s)
*/
int Screen::go_left(int nb)
{
  while(nb>0)
    {
      if((write(fd,"\033[D",3))==-1)
	return -1;
      nb--;
    }
  return 0;
}

/*
  Name : clear_area
  Finality : delete a defined area on the screen
  Decription : delete from #xs to #xe on the lines from #ys to #ye
*/
void Screen::clear_area(int xs,int xe,int ys,int ye)
{
  int size=xe-xs;
  int i=0,j=ys;
  
  init_home_cursor();
  go_down(ys);
  go_right(xs);
  while(j<=ye)
    {
      for(i=0;i<size;i++)
	{
	  switch (j) {
	  case 0:
	    line0[i]=' ';
	    break;
	  case 1:
	    line1[i]=' ';
	    break;
	  case 2:
	    line2[i]=' ';
	    break;
	  case 3:
	    line3[i]=' ';
	    break;
	  }
	}
      init_home_cursor();
      go_right(xs);
      j++;
      go_down(j);
    }
  show();
}

